import multiprocessing
import tqdm


class MultiProcessing:
    def __init__(self, process, total=None, nprocs=multiprocessing.cpu_count()):
        self.process = process
        self.total = total
        self.nprocs = nprocs

    @staticmethod
    def to_list(val, length):
        if isinstance(val, list):
            assert len(val) == length, f'{len(val)} != {length}'
        else:
            val = [val] * length
        return val

    def run(self, *args):
        assert len(args), 'The number of arguments can not be 0'
        args_ = []
        if self.total is None:
            assert isinstance(args[0], list), 'Can not infer the number of expected iterations'
            self.total = len(args[0])
        for arg in args:
            arg = self.to_list(arg, self.total)
            args_.append(arg)
        with multiprocessing.Pool(self.nprocs) as p:
            retval = list(tqdm.tqdm(p.imap(self.process, zip(*args_)), total=self.total, desc=self.process.__name__))
        return retval



import os
import glob
import imageio
import argparse


def image2gif(image_file_list, gif_file, duration):
    assert len(image_file_list), 'There is no image'
    images = [imageio.imread(image_file) for image_file in image_file_list]
    duration /= len(image_file_list) - 1
    imageio.mimsave(gif_file, images, 'GIF', duration=duration)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of image files')
    parser.add_argument('-o', '--output', type=str, help='output gif file')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-d', '--duration', type=float, default=1, help='duration')
    opts = parser.parse_args()

    opts.input = os.path.abspath(opts.input)
    opts.output = opts.input + '.gif' if opts.output is None else opts.output
    print(opts)

    image_path = opts.input
    gif_file = opts.output
    postfix = opts.postfix
    duration = opts.duration
    image_file_list = sorted(glob.glob(os.path.join(image_path, '*.' + postfix)))
    image2gif(image_file_list, gif_file, duration)


if __name__ == '__main__':
    main()



import os
import cv2
import glob

import numpy as np
import tqdm
import argparse


def concat_image(image_file_list1, image_file_list2, output_path, padding_size=5, hstack=True, black=True):
    assert len(image_file_list1) == len(image_file_list2), f'The number of images is not equal ' \
                                                           f'{len(image_file_list1)} vs {len(image_file_list2)}'
    os.makedirs(output_path, exist_ok=True)
    color = 0 if black else 255
    for i in tqdm.trange(len(image_file_list1)):
        image_file1 = image_file_list1[i]
        image_file2 = image_file_list2[i]
        image1 = cv2.imread(image_file1)
        image2 = cv2.imread(image_file2)
        shape = list(image1.shape)
        if hstack:
            shape[1] = padding_size
            padding = np.ones(shape) * color
            image = np.hstack((image1, padding, image2))
        else:
            shape[0] = padding_size
            padding = np.ones(shape) * color
            image = np.vstack((image1, padding, image2))
        image_file = os.path.join(output_path, os.path.basename(image_file1))
        cv2.imwrite(image_file, image)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-a', '--input_path1', type=str, default='.', help='path of input image files')
    parser.add_argument('-b', '--input_path2', type=str, help='another path of input image files', required=True)
    parser.add_argument('-o', '--output_path', type=str, help='path of output image files', required=True)
    parser.add_argument('-p', '--padding_size', type=int, default=5, help='padding size')
    parser.add_argument('-v', '--vertical', dest='hsatck', action='store_false', help='vertical stack')
    parser.add_argument('-w', '--white', dest='black', action='store_false', help='padding with white')
    opts = parser.parse_args()
    print(opts)

    input_path1 = opts.input_path1
    input_path2 = opts.input_path2
    output_path = opts.output_path
    hsatck = opts.hsatck
    padding_size = opts.padding_size
    black = opts.black
    image_file_list1 = sorted(glob.glob(os.path.join(input_path1, '*')))
    image_file_list2 = sorted(glob.glob(os.path.join(input_path2, '*')))
    concat_image(image_file_list1, image_file_list2, output_path, padding_size, hsatck, black)


if __name__ == '__main__':
    main()



import os
import cv2
import glob
import tqdm
import argparse


def images2video(image_file_list, video_file, fourcc, fps, frameSize=None):
    assert len(image_file_list), 'There is no image'
    if frameSize is None:
        image = cv2.imread(image_file_list[0])
        frameSize = (image.shape[1], image.shape[0])
    frameSize = tuple(frameSize)
    videoWriter = cv2.VideoWriter(video_file, fourcc, fps, frameSize)
    assert videoWriter.isOpened(), f'Failed to create file: {video_file}'

    for image_file in tqdm.tqdm(image_file_list, desc=video_file):
        image = cv2.imread(image_file)
        if frameSize != (image.shape[1], image.shape[0]):
            image = cv2.resize(image, tuple(frameSize))
        videoWriter.write(image)
    videoWriter.release()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of image files')
    parser.add_argument('-o', '--output', type=str, help='output video file')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-f', '--fps', type=float, default=29.97, help='desired fps for video')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired frame size for video')
    opts = parser.parse_args()
    print(opts)

    fourcc = cv2.VideoWriter.fourcc('m', 'p', '4', 'v')
    fps = opts.fps
    size = opts.size
    image_file_list = sorted(glob.glob(os.path.join(opts.input, f'*.{opts.postfix}')))
    if len(image_file_list) > 0:
        video_file = os.path.abspath(opts.input) + '.mp4' if opts.output is None else opts.output
        images2video(image_file_list, video_file, fourcc, fps, size)
    else:
        image_path_list = sorted(glob.glob(os.path.join(opts.input, '*/')))
        for image_path in image_path_list:
            video_file = os.path.abspath(image_path) + '.mp4'
            image_file_list = sorted(glob.glob(os.path.join(image_path, f'*.{opts.postfix}')))
            images2video(image_file_list, video_file, fourcc, fps, size)


if __name__ == '__main__':
    main()



import os
import shutil
import argparse
import glob
from image2pdf import image2pdf
from pdf2image import pdf2image


def compress_pdf(input_file, output_file, temp_path, zoom):
    assert not os.path.exists(temp_path), f'temp_path must not exist'
    pdf2image(input_file, None, temp_path, zoom)
    image_file_list = sorted(glob.glob(os.path.join(temp_path, '*.png')))
    image2pdf(image_file_list, output_file)
    shutil.rmtree(temp_path)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, help='input pdf file', required=True)
    parser.add_argument('-o', '--output', type=str, help='output pdf file', required=True)
    parser.add_argument('-t', '--temp_path', type=str, help='temporary path for image files')
    parser.add_argument('-z', '--zoom', type=float, default=1, help='zoom scale')
    opts = parser.parse_args()

    opts.temp_path = os.path.splitext(opts.input)[0] if opts.temp_path is None else opts.temp_path
    print(opts)

    input_file = opts.input
    output_file = opts.output
    temp_path = opts.temp_path
    zoom = opts.zoom
    compress_pdf(input_file, output_file, temp_path, zoom)


if __name__ == '__main__':
    main()



import os
import argparse
import fitz
import glob


def image2pdf(image_file_list, pdf_file):
    assert len(image_file_list), 'There is no image'
    doc = fitz.open()
    for image_file in image_file_list:
        imgdoc = fitz.open(image_file)
        pdfbytes = imgdoc.convert_to_pdf()
        imgpdf = fitz.open('pdf', pdfbytes)
        doc.insert_pdf(imgpdf)
    doc.save(pdf_file)
    doc.close()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='input image file or path of image files')
    parser.add_argument('-o', '--output', type=str, help='output pdf file')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename (valid only if '
                                                                         'input is a path)')
    opts = parser.parse_args()

    if os.path.isdir(opts.input):
        opts.output = os.path.abspath(opts.input) + '.pdf' if opts.output is None else opts.output
        image_file_list = sorted(glob.glob(os.path.join(opts.input, f'*.{opts.postfix}')))
    else:
        opts.output = os.path.splitext(opts.input)[0] + '.pdf' if opts.output is None else opts.output
        image_file_list = [opts.input]

    print(opts)
    pdf_file = opts.output
    image2pdf(image_file_list, pdf_file)


if __name__ == '__main__':
    main()



import os
import fitz
import argparse


def pdf2image(pdf_file, image_file, image_path, zoom=1):
    pdf = fitz.open(pdf_file)
    trans = fitz.Matrix(zoom, zoom)

    save_to_folder = pdf.page_count > 1 or image_file is None
    if save_to_folder:
        os.makedirs(image_path, exist_ok=True)

    for index in range(pdf.page_count):
        page = pdf[index]
        pm = page.get_pixmap(matrix=trans)
        if save_to_folder:
            image_file = os.path.join(image_path, f'{index + 1:06d}.png')
        pm.save(image_file)
    pdf.close()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, help='input pdf file', required=True)
    parser.add_argument('-o', '--output', type=str, help='output image file')
    parser.add_argument('-z', '--zoom', type=float, default=1, help='zoom image')
    opts = parser.parse_args()

    opts.output = os.path.splitext(opts.input)[0] + '.png' if opts.output is None else opts.output
    print(opts)

    pdf_file = opts.input
    image_file = opts.output
    image_path = os.path.splitext(image_file)[0]
    zoom = opts.zoom
    pdf2image(pdf_file, image_file, image_path, zoom)


if __name__ == '__main__':
    main()



import os
import time
import psutil
import argparse


def startup(pid, period, cmd):
    while psutil.pid_exists(pid):
        time.sleep(period)

    print(f'Start up: {cmd}')
    os.system(cmd)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--pid', type=int, help='pid of waited process')
    parser.add_argument('-c', '--cmd', type=str, help='cmd to be executed')
    parser.add_argument('-p', '--period', type=int, default=600, help='monitoring period')
    opts = parser.parse_args()
    print(opts)

    startup(pid=opts.pid, period=opts.period, cmd=opts.cmd)


if __name__ == '__main__':
    main()



import os

import xlrd
import argparse


def get_cell_font_bold(workbook, cell):
    xf_index = cell.xf_index
    xf_style = workbook.xf_list[xf_index]
    font_index = xf_style.font_index
    font = workbook.font_list[font_index]
    return font.bold


def excel2latex(excel_file, text_file, sheet_index=0):
    workbook = xlrd.open_workbook(excel_file, formatting_info=True)
    sheet = workbook.sheet_by_index(sheet_index)

    with open(text_file, 'w') as f:

        precisions = []
        for j in range(sheet.ncols):
            max_precision = 0
            for i in range(sheet.nrows):
                if sheet.cell_type(i, j) == 2:
                    value = sheet.cell_value(i, j)
                    if value == round(value):
                        precision = 0
                    else:
                        precision = len(str(value).split('.')[-1])
                    max_precision = max(max_precision, precision)
            precisions.append(max_precision)

        for i in range(sheet.nrows):
            for j in range(sheet.ncols):
                cell = sheet.cell(i, j)
                type = cell.ctype
                value = cell.value
                bold = get_cell_font_bold(workbook, cell)
                if type == 2:
                    precision = precisions[j]
                    value = f'%.{precision}f' % value
                if bold:
                    f.write('\\textbf{' + str(value) + '}')
                else:
                    f.write(str(value))
                if j == sheet.ncols - 1:
                    f.write(' \\\\\n')
                else:
                    f.write(' & ')


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, help='input excel file', required=True)
    parser.add_argument('-o', '--output', type=str, help='output text file')
    parser.add_argument('-s', '--sheet_index', type=int, default=0, help='sheet index')
    opts = parser.parse_args()

    opts.output = os.path.splitext(opts.input)[0] + '.txt' if opts.output is None else opts.output
    print(opts)

    excel_file = opts.input
    text_file = opts.output
    sheet_index = opts.sheet_index
    excel2latex(excel_file, text_file, sheet_index)


if __name__ == '__main__':
    main()



import os
import cv2
import glob
import tqdm
import argparse


def image2image(image_file_list, postfix, image_size=None):
    for image_file in tqdm.tqdm(image_file_list):
        new_image_file = os.path.splitext(image_file)[0] + f'.{postfix}'
        image = cv2.imread(image_file)
        if image_size is not None:
            image = cv2.resize(image, tuple(image_size))
        cv2.imwrite(new_image_file, image)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-p', '--path', type=str, default='.', help='path of image files')
    parser.add_argument('-i', '--input_postfix', type=str, default='png', help='original postfix of image filename')
    parser.add_argument('-o', '--output_postfix', type=str, default='jpg', help='desired postfix of image filename')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired image size')
    opts = parser.parse_args()
    print(opts)

    image_path = opts.path
    postfix = opts.output_postfix
    image_size = opts.size
    image_file_list = sorted(glob.glob(os.path.join(image_path, f'*.{opts.input_postfix}')))
    image2image(image_file_list, postfix, image_size)


if __name__ == '__main__':
    main()



import os
import cv2
import glob
import tqdm
import argparse
import time


def play_images(image_file_list, fps):
    period = 1.0 / fps
    for image_file in tqdm.tqdm(image_file_list):
        start_time = time.time()
        image = cv2.imread(image_file)
        cv2.namedWindow('image', 0)
        cv2.imshow('image', image)
        current_time = time.time()
        delta_time = current_time - start_time
        sleep_time = max(1, int((period - delta_time) * 1000))
        cv2.waitKey(sleep_time)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='path of image files')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-f', '--fps', type=float, default=29.97, help='desired fps for video')
    opts = parser.parse_args()
    print(opts)

    fps = opts.fps
    image_file_list = sorted(glob.glob(os.path.join(opts.input, f'*.{opts.postfix}')))
    play_images(image_file_list, fps)


if __name__ == '__main__':
    main()



import os
import cv2
import glob
import tqdm
import argparse


def video2images(video_file, image_path, postfix, frameSize=None):
    os.makedirs(image_path, exist_ok=True)
    videoCapture = cv2.VideoCapture(video_file)
    assert videoCapture.isOpened(), f'Failed to open file: {video_file}'

    frame_count = int(videoCapture.get(cv2.CAP_PROP_FRAME_COUNT))
    for index in tqdm.trange(frame_count, desc=video_file):
        status, image = videoCapture.read()
        if not status:
            break
        if frameSize is not None:
            image = cv2.resize(image, tuple(frameSize))
        image_file = os.path.join(image_path, f'{index + 1:06d}.{postfix}')
        cv2.imwrite(image_file, image)

    videoCapture.release()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', type=str, default='.', help='video file or path of video files')
    parser.add_argument('-o', '--output', type=str, help='path of image files')
    parser.add_argument('-p', '--postfix', type=str, default='png', help='postfix of image filename')
    parser.add_argument('-s', '--size', type=int, nargs=2, help='desired frame size for image')
    opts = parser.parse_args()
    print(opts)

    postfix = opts.postfix
    size = opts.size
    if os.path.isdir(opts.input):
        video_file_list = sorted(glob.glob(os.path.join(opts.input, '*.mp4')))
        for video_file in video_file_list:
            image_path = os.path.splitext(video_file)[0]
            video2images(video_file, image_path, postfix, size)

    elif os.path.isfile(opts.input):
        video_file = opts.input
        image_path = os.path.splitext(video_file)[0] if opts.output is None else opts.output
        video2images(video_file, image_path, postfix, size)


if __name__ == '__main__':
    main()
